% QuNacsolve is the template line search optimization method for solving
%     min  f_eval(x)
%  It initiates by calling method_boot(...).
% At each iteration, a descent direction is generated by calling d= descent_method(...). A line search is called and a
% step is calculated.
% Copyright (c) 2014.  Robert Gower.
function [out] =  QuNacsolve(x0, f_eval, g_eval, Hess_opt, method_boot, descent_method,   opts)
if feature('numThreads') > 1
    tic; % wall clock time.
else
    tstart = cputime; % CPU time.
end

grad_cur  = g_eval(x0);
normg0 = norm(grad_cur );
% This changes x0 and takes can take a grad step!
if(~isempty(method_boot))
    method_boot(x0,f_eval, g_eval, Hess_opt,opts);
end
x = x0;
iteration =0;
error = 1;
tol = opts.tol;
xdiff = 2*tol^2;
total_memory= 0;
if opts.prnt
    fprintf('%s', DATA.name);
    head = '        f         |grad|/|grad0|    Itr     stp    cgstp    mem';
    form = '%12.6e     %12.6e  %5.0f %9.2g %5.0f   %5.0f\n';
    disp('  ');   disp(head);
    if(isfield(opts,'PCG') )
        fprintf(form, f_eval(x0), 1, 0,1, opts.PCG.maxit, opts.PCG.memory_limit);
    else
        fprintf(form, f_eval(x0), 1, 0,1, 0, 1);
    end
end

% Information for plotting
if(opts.plotting)
    fvals =f_eval(x);
    errors =1;
    times = 0;
end
%% Generic Line Search Method
while (error > tol && xdiff > tol^2 )
    d = descent_method(x, g_eval, Hess_opt ,tol, opts);
    d = d(:);
    grad_cur =  g_eval(x);
    
    if(grad_cur'*d >0)
        display('NOT DESCENT! Switching direction')
        d = -d;
    end
    stp = line_search(opts.line_search,x,f_eval,g_eval, grad_cur, d,Hess_opt);
    x = x+stp*d;
    DATA.delta = stp*d;
    DATA.stp = stp;
    xdiff = norm(stp*d);
    grad_cur =  g_eval(x);
    f = f_eval(x);
    error = norm(grad_cur)/normg0;
    % Timing iteration
    if feature('numThreads') > 1
        cur_time = toc;
    else
        tend = cputime;
        cur_time = tend - tstart;
    end
    % Information for plotting
    if(opts.plotting)
        errors(end+1) = error;
        times(end+1) = cur_time;
        fvals(end+1) = f;
    end
    % ------------%
    iteration = iteration+1;
    if opts.prnt
        if(isfield(opts,'PCG') &&  isfield(opts.PCG,'CGmem'))
            %  if(isfield(opts.PCG,'CGmem'))
            total_memory = total_memory+opts.PCG.CGmem;
            fprintf(form, f, error, iteration,xdiff, opts.PCG.CGsteps, total_memory);
        else
            fprintf(form, f, error, iteration,xdiff, 0, total_memory);
        end
    end
    
    if feature('numThreads') > 1
        cur_time = toc;
    else
        tend = cputime;
        cur_time = tend - tstart;
    end
    
    if(cur_time > opts.Timeout)
        break;
    end
    
end % End of solver
%%
% Time the finish, remainder is for testing
if feature('numThreads') > 1
    out.cput = toc;
else
    tend = cputime;
    out.cput = tend - tstart;
end

if(xdiff<= tol^2 && error >tol)
    out.stopping_flag = 'smallstep';
elseif (cur_time > opts.Timeout)
    out.stopping_flag = 'timeout';
elseif (isnan(error) || isnan(xdiff))
    out.stopping_flag = 'NaN';
else
    out.stopping_flag = 'conv.';
end

out.name = DATA.name;
out.error = error;
out.iteration = iteration;
out.x = x;
out.f = f;

if(opts.plotting)
out.fvals =fvals;
out.times =times;
out.errors =errors;
end
%% Test the output
if(strcmp(opts.problem,'logistic'))
    out.accuracy=  accurary_prediction( opts.X, opts.y, x);
 %   out.val_accuracy = test_log_model(opts, out);
    opts.X = [];
    opts.y = [];
end
out.memory_limit = opts.PCG.memory_limit;
if(isfield(opts,'PCG') )
    opts.PCG = [];
end
if(isfield(opts,'x0') )
    opts.x0 = [];
end
out.opts = opts;

%% Print conclusion
fprintf('%s total time %12.6f s', out.name,out.cput);
head = '        f         |grad|/|grad0|    Itr     stp    cgstp    mem';
form = '%12.6e     %12.6e  %5.0f %9.2g %5.0f   %5.0f\n';
disp('  ');   disp(head);
if(isfield(opts,'PCG') &&  isfield(opts.PCG,'CGmem'))
    %  if(isfield(opts.PCG,'CGmem'))
    total_memory = total_memory+opts.PCG.CGmem;
    fprintf(form, f, error, iteration,xdiff, opts.PCG.CGsteps, total_memory);
else
    fprintf(form, f, error, iteration,xdiff, 0, total_memory);
end
end